@@ -404,7 +404,7 @@ c              until current partile dies or all are put into stack.
 #include  "Zcnfig.h"
 
 
-      integer icon
+      integer icon, info
 
       do while (.true.)
 c                current track is in  cTrack
@@ -412,11 +412,17 @@ c          fix process and free path
 c          and compute tentative new pos.--> Move
 
          call epnewp(icon)
-         if(icon .ne. 0) goto 100 !  very rare inconsisty neglect.
+         if(icon .ne. 0) goto 100 ! abort is specified by the user
 c          energy loss count in user hook
          if(Det.cmp(Cn).CountDE .gt. 0 .and.
      *           cTrack.p.charge .ne. 0) then
-            call userde(0, cTrack, Move, Media(MediaNo))
+            if( Move.Track.p.fm.p(4)-Move.Track.p.mass
+     *                                     .le. KEmin ) then 
+               info = 1
+            else
+               info = 0
+            endif
+            call userde(info, cTrack, Move, Media(MediaNo))
          endif
 c          add  time  
          if(TimeStruc) then
@@ -458,6 +464,12 @@ c                       icon =  1 always.
             elseif(Move.Cross) then
                call epCross(icon)
             endif
+         else
+c              take trace info. of dead particle.
+            if(Trace) then
+               call epTrace
+            endif
+
          endif
          if         (icon .ne. 0)
      *                       goto 100
@@ -956,7 +968,8 @@ c
       if(E .le. EupperBndCS) then
          call epcompp(Media(MediaNo), E, prob, tcomp)
          if(Photo) then
-            call epphotoEp(Media(MediaNo).pe,  E, prob, tphot)
+c            call epphotoEp(Media(MediaNo).pe,  E, prob, tphot)
+            call epphotoEp(Media(MediaNo),  E, prob, tphot)  ! v8.0
          else
             tphot = 1.e35
          endif
@@ -1047,6 +1060,10 @@ c             sample path for brems
             Move.proc='anih'
          endif
       endif
+      Move.dt = t
+      Move.dx = Move.dt * Media(MediaNo).X0g
+      Move.dl = Move.dx * Media(MediaNo).gtocm
+
 c                     only if X0 > 10 km; may be 30 km is o.k
       if(Sync .eq.  2 .and. Media(MediaNo).X0 .gt. 10.d5 ) then
 c          sample synchrotron emission path
@@ -1059,10 +1076,7 @@ c          sample synchrotron emission path
             Move.dl = dl
             Move.proc = 'sync'
          endif
-      else
-         Move.dt = t
-         Move.dx = Move.dt * Media(MediaNo).X0g
-         Move.dl = Move.dx * Media(MediaNo).gtocm
+c      else
       endif
       end
 c     ******************
@@ -1651,14 +1665,6 @@ c            the other electron energy
 c         sample angle; smaller enery electron must be put
 c         last
        call epPairAng(e2, masele, teta2) ! teta2 < pi/2
-       teta1=teta2 * e2/e1
-       if(teta1 .lt. 0.03d0) then
-          cos1 = 1. - teta1**2/2
-          sin1 = teta1
-       else
-          cos1 = cos(teta1)
-          sin1 = sin(teta1)
-       endif
        if(teta2 .lt. 0.03d0) then
           cos2 = 1. - teta2**2/2
           sin2 = teta2
@@ -1667,6 +1673,24 @@ c         last
           sin2 = sin(teta2)
        endif
 
+c
+       sin1 = sin2 * sqrt(  (e2**2-masele**2)/(e1**2-masele**2) )
+       if(sin1 .lt. 0.03d0) then
+          cos1 = 1.- sin1**2/2
+       else
+          cos1 = sqrt(1.d0 - sin1**2)
+       endif
+c
+c          the next simplified treatment is also no problem.
+c       teta1=teta2 * e2/e1
+c       if(teta1 .lt. 0.03d0) then
+c          cos1 = 1. - teta1**2/2
+c          sin1 = teta1
+c       else
+c          cos1 = cos(teta1)
+c          sin1 = sin(teta1)
+c       endif
+
        elec1 = cTrack           ! copy everything first
 c               sample direction cos. of 1st
 
@@ -1709,7 +1733,8 @@ c      ************
        logical kbtest
 c
 c           get Photo-electron energy
-       call epphotoEe(Media(MediaNo).pe, 
+c       call epphotoEe(Media(MediaNo).pe,   < v8.0
+       call epphotoEe(Media(MediaNo),
      *      cTrack.p.fm.p(4), eout)
        if(kbtest(Eabsorb, BitPhotoElec)) then
 c            energy absorbed by atom is Eabs = Eshell= Eg-(Ee-Me)
